import sys, webview

def window(title='window',location='http://google.com'):
	
	webview.create_window(title,location, resizable=True,width=480,height=720);

if __name__=='__main__':

	if(len(sys.argv)==3):
		window(sys.argv[1],sys.argv[2])
	else:
		print 'using http://localhost:6432 as view'
		window('bandwidth monitor',location='http://localhost:6432/index.html')


from distutils.core import setup
from setuptools import find_packages

setup(
    # Application name:
    name="hogwatch",

    # Version number (initial):
    version="0.0.1",

    # Application author details:
    author="Akshay Kumar",
    author_email="akshay.kmr4321@gmail.com",

    # Packages
    packages=find_packages(),
    scripts=["hogwatch"],

    # Include additional files into the package
    include_package_data=True,

    # Details
    url="https://github.com/akshayKMR/hogwatch",
    setup_requires=['setuptools-markdown'],
    long_description_markdown_filename='README.md',

    #
    license="GNU GPL V3",
    description="find out whats hogging your internet",

    long_description="A bandwidth monitor that shows per process network transfer",

    # Dependent packages (distributions)
    install_requires=[
        "bottle",
        'pywebview',
        'gevent-websocket',
	    'netifaces'
    ],
)



import subprocess, sys
from Queue import Queue
from pprint import pprint
from threading import Event
from decimal import *
import time

#todo learn logging

class NethogsWatchdog :
    def __init__(self,debug=False,devices=[],delay=1):

        from sys import platform as _platform
        if _platform == "linux" or _platform == "linux2":
            pass
            # linux
        elif _platform == "darwin":
            if len(devices)==0:
                devices=['en0']
        #elif _platform == "win32":
        self.devices=devices
        self.delay=str(delay)
        self.debug=debug

    def watch_transfer(self,mode='transfer_rate',bridge={}):
        #param 0=rate, 3 amount in MB

        if mode not in ['transfer_rate','transfer_amount']:
            raise ValueError('mode not supported')

        if mode=='transfer_rate':   
            param='0'
        else:
            param='1'

        cmd=['nethogs','-d',self.delay, '-v',param,'-t']+self.devices
        if self.debug:
            print cmd
        p = subprocess.Popen(cmd, stdout=subprocess.PIPE, bufsize=1)

        refresh_flag=True #naming is tough...
        report={}
        report['mode']=mode
        report['ctr']=0 

        entries=[]      
        for line in iter(p.stdout.readline, b''):
            #print line
            if(line.find('Refreshing')==-1):

                if refresh_flag:
                    continue

                split=line.split()
                if(len(split)!=3):
                    continue

                entry={}
                entry['process']=split[0]


                if(mode=='transfer_rate'):
                    #kbps out/in
                    entry['kbps_out']=round(Decimal((float(split[1]))),1)
                    entry['kbps_in']=round(Decimal((float(split[2]))),1)
                else: #mode is 'transfer_amount'
                    #MB out/in
                    entry['kb_out']=round(Decimal((float(split[1]))),1)
                    entry['kb_in']=round(Decimal((float(split[2]))),1)

                entries.append(entry)
            else:
                if refresh_flag:
                    refresh_flag=False 
                    continue

                if(len(entries)==0):
                    continue
                
                # print '\n'
                # pprint(entries)
                total_in,total_out= 0,0
                for entry in entries:
                    if(report['mode']=='transfer_rate'):
                        total_in+=entry['kbps_in']
                        total_out+=entry['kbps_out']
                    else:
                        total_in+=entry['kb_in']
                        total_out+=entry['kb_out']

                report['total_in']=total_in
                report['total_out']=total_out
                report['entries']=entries
                report['ctr']+=1 # I dont know why there is a race condition here -_-'
                entries=[]

                if self.debug:
                    pprint(report)
                else:
                    report['timestamp']=int(round(time.time() * 1000))
                    bridge['queue'].put(report)

        p.stdout.close()

        if self.debug:
            p.wait()
        else:
            bridge['event'].wait()


if __name__=='__main__':

    x=NethogsWatchdog(debug=True,devices=sys.argv[1:])
    x.watch_transfer()


import json, gevent
from Queue import Queue
from threading import Thread, Event
from pprint import pprint
import netifaces
import os

from bottle import (
    Bottle,
    route,
    run,
    template,
    static_file,
    redirect,
    debug,
    request,
    abort
)
local_path=os.path.abspath(os.path.dirname(__file__))
path_for_static_assets= os.path.join(local_path,'static')
# local_path=os.path.dirname(os.path.abspath('web_service.py'))
# from sys import platform as _platform
# if _platform == "linux" or _platform == "linux2":
#     path_for_static_assets=os.path.join(local_path,'application','server','static')
# elif _platform == "darwin":
#     path_for_static_assets=os.path.join(local_path,'application','server','static')
# elif _platform == "win32":
#     pass

path_for_static_assets += '/'
print path_for_static_assets

from watchdogs.nethogs import NethogsWatchdog

def app_server(
        host='0.0.0.0',
        port=6432,
        debugMode=True,
        reloader=True,
        ws=True
    ):
    app=Bottle()

    @app.route('/')
    def home():
        redirect('/index.html')

    @app.route('/interfaces')
    def interfaces():
        return {'interfaces': netifaces.interfaces()}
    
    if ws:
        connections_count=0

        @app.route('/websocket/<device>/<mode>')
        def handle_websocket(device='all',mode='transfer_rate'):
            if(device=='all'):
                device_list=[]
            else:
                device_list= device.split('_')

            if debugMode:
                pprint(device_list)
                pprint(mode)
                print '******************************************'

            tommy=NethogsWatchdog(devices=device_list)
            bridge={
                'queue': Queue(),
                'event': Event()
            }

            t=Thread(
                target=tommy.watch_transfer,
                args=(mode,bridge)
            )
            t.daemon=True
            t.start()

            wsock = request.environ.get('wsgi.websocket')
            if not wsock:
                abort(400, 'Expected WebSocket request.')
            
            while True:
                try:
                    message = wsock.receive()
                    report=bridge['queue'].get()
                    wsock.send(json.dumps(report))
                    gevent.sleep(0.1)
                except WebSocketError:
                    bridge['event'].set()
                    break


    @app.route('/<filename:path>')
    def send_static(filename):
        
        return static_file(
            filename,
            root=path_for_static_assets
        )

    if ws:
        from gevent.pywsgi import WSGIServer
        from geventwebsocket import WebSocketError
        from geventwebsocket.handler import WebSocketHandler
        server = WSGIServer((host, port), app, handler_class=WebSocketHandler)
        server.serve_forever()
    else:
        port=str(port)
        app.run(host=host, port=port, debug=debugMode, reloader=reloader)



if __name__ == '__main__':
    app_server(ws=True)
    #todo kill threads on kill signal